#ifndef __TEST_UTILS_H__
#define __TEST_UTILS_H__

#define THEN_WHEN(x)
#define THEN_CHECK(x)

#include <initializer_list>
#include <memory>
#include <string>
#include <vector>

#include "base-utils/yaml.h"
#include "config/path.h"
#include "config/pattern.h"
#include "plugins/executionContext.h"
#include "plugins/pluginUtils.h"

#include "executorStub.h"

namespace execHelper {
namespace config {
class SettingsNode;
} // namespace config

namespace test::baseUtils {
class ConfigFileWriter;
} // namespace test::baseUtils
} // namespace execHelper

namespace std {
template <typename T>
auto operator<<(std::ostream& os, const std::vector<T>& stream) noexcept -> std::ostream& {
    for(const auto& element : stream) {
        os << element << ", ";
    }
    return os;
}

template <typename T, typename U>
auto operator<<(std::ostream& os, const std::pair<T, U>& stream) noexcept -> std::ostream& {
    os << stream.first << ": " << stream.second;
    return os;
}
} // namespace std

namespace execHelper::test::utils {
using Patterns = std::vector<config::Pattern>;

using Arguments = std::vector<std::string>;
struct MainVariables {
    int argc;
    std::unique_ptr<char*[]> argv;

    explicit MainVariables(const Arguments& arguments);
};

template <typename T> void appendVectors(T& appendTo, const T& appendFrom) {
    appendTo.insert(std::end(appendTo), std::begin(appendFrom),
                    std::end(appendFrom));
}

baseUtils::YamlWriter toYaml(const config::SettingsNode& settings,
                             const config::Patterns& patterns) noexcept;
void writeSettingsFile(
    baseUtils::ConfigFileWriter& configFileWriter,
    const config::SettingsNode& settings,
    const config::Patterns& patterns) noexcept;
std::string convertToConfig(const Patterns& patterns) noexcept;
std::string
convertToConfig(const config::SettingsNode& rootSettings,
                const std::string& prepend = std::string()) noexcept;
std::string
convertToConfig(const config::SettingsNode& settings, const Patterns& patterns,
                const std::string& prepend = std::string()) noexcept;

std::string convertToConfig(std::string key, std::string value,
                            const std::string& prepend = std::string());
std::string convertToConfig(const std::string& key,
                            const std::initializer_list<std::string>& values,
                            const std::string& prepend = std::string());
std::string convertToConfig(const std::string& key,
                            const std::vector<std::string>& values,
                            const std::string& prepend = std::string());

std::string
convertToConfig(const std::initializer_list<std::string>& keys,
                const std::string& value,
                const std::string& prepend = std::string()) noexcept;
std::string
convertToConfig(const std::initializer_list<std::string>& keys,
                const std::initializer_list<std::string>& values,
                const std::string& prepend = std::string()) noexcept;
std::string
convertToConfig(const std::initializer_list<std::string>& keys,
                const std::vector<std::string>& values,
                const std::string& prepend = std::string()) noexcept;
std::string
convertToConfig(const std::vector<std::string>& keys,
                const std::initializer_list<std::string>& values,
                const std::string& prepend = std::string()) noexcept;
std::string
convertToConfig(const std::vector<std::string>& keys,
                const std::vector<std::string>& values,
                const std::string& prepend = std::string()) noexcept;
std::string basename(const std::string& file);

config::PatternCombinations createPatternCombination(
    const std::initializer_list<config::PatternKey>& keys,
    const std::initializer_list<config::PatternValue>& values) noexcept;

config::PatternCombinations
createPatternCombination(const config::PatternKeys& keys,
                         const config::PatternValues& values) noexcept;
plugins::PatternPermutator
makePatternPermutator(const config::Patterns& patterns) noexcept;
core::test::ExecutorStub::TaskQueue
getExpectedTasks(const core::Task& task,
                 const config::Patterns patterns) noexcept;
core::test::ExecutorStub::TaskQueue
getExpectedTasks(const core::test::ExecutorStub::TaskQueue& tasks,
                 const config::Patterns patterns) noexcept;

std::string toString(const config::SettingsNode& settings,
                     unsigned int nbOfTabs = 0) noexcept;

auto getPredefinedPatterns(const config::Path& rootDirectory) noexcept -> Patterns;
} // namespace execHelper::test::utils

#endif /* __TEST_UTILS_H__ */
